//===-- Count_Blocks_in_Loops.cpp - How to use the LoopInfo analysis ------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// This pass counts the number of basic blocks per loop in a program. It is an
// example about how to use the LoopInfo pass to obtain information about
// loops. It only goes over the outermost loops. The next example, e.g.,
// Count_Blocks_in_Loops2 fixes this problem.
//
// Author: Fernando.
// Date: July 20th, 2014.
// Usage: opt -load dcc888.dylib -bbloop -disable-output loop.bc
//
//===----------------------------------------------------------------------===//

#include "llvm/IR/Function.h"        //Function info
#include "llvm/Analysis/LoopPass.h"  //LoopBase info

using namespace llvm;
using namespace std;

namespace {
  struct BBinLoops : public FunctionPass {
    static char ID;
    BBinLoops() : FunctionPass(ID) {}

    void getAnalysisUsage(AnalysisUsage &AU) const {
      AU.addRequired<LoopInfo>();
      AU.setPreservesAll();
    }

    virtual bool runOnFunction(Function &F) {
      LoopInfo &LI = getAnalysis<LoopInfo>();
      int loopCounter = 0;
      errs() << "Function " << F.getName() + "\n";
      for (LoopInfo::iterator i = LI.begin(), e = LI.end(); i != e; ++i) {
        Loop *L = *i;
        int bbCounter = 0;
        loopCounter++;
        for(Loop::block_iterator bb = L->block_begin(); bb != L->block_end();
            ++bb) {
          bbCounter+=1;
        }
        errs() << "Loop ";
        errs() << loopCounter;
        errs() << ": #BBs = ";
        errs() << bbCounter;
        errs() << "\n";
      }
      return(false);
    }
  };
}

char BBinLoops::ID = 0;
static RegisterPass<BBinLoops> X("bbloop",
    "Count the number of BBs inside each loop");
