//===--- Count_Blocks_in_Loops2.cpp - How to use the LoopInfo analysis ----===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// This pass counts the number of basic blocks per loop in a program. It
// traverses all the loops in the program recursively. This is an
// example on how to use the LoopInfo pass to obtain information about loops.
//
// Author: Fernando.
// Date: July 20th, 2014.
// Usage: opt -load dcc888.dylib -bbloop -disable-output loop.bc
//
//===----------------------------------------------------------------------===//

#include "llvm/Support/raw_ostream.h"
#include "llvm/IR/Function.h"        //Function info
#include "llvm/Analysis/LoopPass.h"  //LoopBase info

using namespace llvm;
using namespace std;

namespace {
  struct BBinLoops : public FunctionPass {
    static char ID;
    BBinLoops() : FunctionPass(ID) {}

    void getAnalysisUsage(AnalysisUsage &AU) const {
      AU.addRequired<LoopInfo>();
      AU.setPreservesAll();
    }

    void countBlocksInLoop(Loop *L, unsigned nesting) {
      unsigned numBlocks = 0;
      Loop::block_iterator bb;
      for(bb = L->block_begin(); bb != L->block_end();++bb) {
        numBlocks++;
      }
      errs() << "Loop level " << nesting << " has " << numBlocks << " blocks\n";
      vector<Loop*> subLoops = L->getSubLoops();
      Loop::iterator j, f;
      for (j = subLoops.begin(), f = subLoops.end(); j != f; ++j) {
        countBlocksInLoop(*j, nesting + 1);
      }
    }

    virtual bool runOnFunction(Function &F) {
      LoopInfo &LI = getAnalysis<LoopInfo>();
      errs() << "Function " << F.getName() + "\n";
      for (LoopInfo::iterator i = LI.begin(), e = LI.end(); i != e; ++i) {
        countBlocksInLoop(*i, 0);
      }
      return(false);
    }
  };
}

char BBinLoops::ID = 0;
static RegisterPass<BBinLoops> X("bbloop2",
    "Count the number of BBs inside each loop");


